package hdl

import (
	"applet/app/db"
	"applet/app/e"
	"applet/app/md"
	"applet/app/utils"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/implement"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/model"
	md3 "code.fnuoos.com/EggPlanet/egg_system_rules.git/md"
	md2 "code.fnuoos.com/EggPlanet/egg_system_rules.git/rule/egg_energy/md"
	"encoding/json"
	"fmt"
	"github.com/gin-gonic/gin"
	"github.com/shopspring/decimal"
	"math"
	"strconv"
	"strings"
	"time"
)

// EggEnergyDetails
// @Summary      蛋蛋星球-添加好友-蛋蛋能量明细(获取)
// @Tags         添加好友
// @Description  蛋蛋能量明细(获取)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Param    req    body	md.EggEnergyDetailReq 	true  "分页类型必填"
// @Success      200   {object}   md.EggEnergyDetailResp	"具体数据"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/v1/addFriend/eggEnergyDetails [POST]
func EggEnergyDetails(c *gin.Context) {
	val, exists := c.Get("user")
	if !exists {
		e.OutErr(c, e.ERR_USER_CHECK_ERR, nil)
		return
	}
	user, ok := val.(*model.User)
	if !ok {
		e.OutErr(c, e.ERR_USER_CHECK_ERR, nil)
		return
	}
	var req *md.EggEnergyDetailReq
	if err1 := c.ShouldBindJSON(&req); err1 != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err1.Error())
		return
	}

	//1、查找 `OneCirclesPublicPlatoonBasicSetting` 基础设置
	energyBasicSettingDb := implement.NewEggEnergyBasicSettingDb(db.Db)
	eggEnergyBasicSetting, err := energyBasicSettingDb.EggEnergyBasicSettingGetOneByParams(map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	var coinID int
	isAsc := false
	if req.Kind == "1" { // 待结
		coinID = eggEnergyBasicSetting.PersonEggEnergyCoinId
	} else { // 可用
		coinID = eggEnergyBasicSetting.TeamEggEnergyCoinId
	}
	if req.Asc == "1" {
		isAsc = true
	}
	direction := 0
	switch req.Direction {
	case "1":
		direction = 1
	case "2":
		direction = 2
	default:
		direction = 0
	}
	transferType := 0
	switch req.Type {
	case "1":
		transferType = 1
	case "2":
		transferType = 2
	default:
		transferType = 0
	}
	flowDb := implement.NewUserVirtualCoinFlowDb(db.Db)
	flows, total, err := flowDb.UserVirtualCoinFlowFindByCoinAndUser(req.Page, req.Limit, coinID, user.Id, "", "", direction, isAsc, transferType)
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}

	var list = make([]map[string]string, 0)
	for _, flow := range flows {
		var tmp = map[string]string{
			"title":         flow.Title,
			"time":          flow.CreateAt,
			"type":          utils.IntToStr(flow.Direction),
			"value":         flow.Amount,
			"transfer_type": utils.IntToStr(flow.TransferType),
		}
		list = append(list, tmp)
	}

	//3、获取当前用户 蛋蛋能量(可用+结算)、 预估总价值
	var teamRewardSetting *md2.TeamRewardSettingStruct
	err = json.Unmarshal([]byte(eggEnergyBasicSetting.DirectPushReward), &teamRewardSetting)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
		return
	}
	virtualAmountDb := implement.NewUserVirtualAmountDb(db.Db)
	personEggEnergy, err := virtualAmountDb.GetUserVirtualWalletBySession(user.Id, eggEnergyBasicSetting.PersonEggEnergyCoinId)
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	teamEggEnergy, err := virtualAmountDb.GetUserVirtualWalletBySession(user.Id, eggEnergyBasicSetting.TeamEggEnergyCoinId)
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	coreDataDb := implement.NewEggEnergyCoreDataDb(db.Db)
	coreData, err := coreDataDb.EggEnergyCoreDataGet()
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	totalEggEnergy := utils.StrToFloat64(personEggEnergy.Amount) + utils.StrToFloat64(teamEggEnergy.Amount)
	totalEggEnergyPrice := totalEggEnergy * utils.StrToFloat64(coreData.NowPrice)

	transferTypeList := md.TransferTypeList{
		AvailableIntegralList: md.IntegralList{
			IncomeList: []md.TransferTypeNode{
				{
					Value: strconv.Itoa(md3.EggPlantEggEnergySignInSettlementPersonalRewardForUserVirtualCoinFlow),
					Name:  md3.EggPlantEggEnergySignInSettlementPersonalReward,
				},
				{
					Value: strconv.Itoa(md3.EggPlantBalanceExchangeForEggEnergyForUserVirtualCoinFlow),
					Name:  md3.EggPlantBalanceExchangeForEggEnergy,
				},
				{
					Value: strconv.Itoa(md3.EggPlantPersonalActiveCoinExchangeEggEnergyForUserVirtualCoinFlow),
					Name:  md3.EggPlantPersonalActiveCoinExchangeEggEnergy,
				},
				{
					Value: strconv.Itoa(md3.EggPlantSettlementEggEnergyExchangeEggEnergyForUserVirtualCoinFlow),
					Name:  md3.EggPlantSettlementEggEnergyExchangeEggEnergy,
				},
			},
			ExpendList: []md.TransferTypeNode{
				{
					Value: strconv.Itoa(md3.EggPlantEggEnergyExchangeForBalanceForUserVirtualCoinFlow),
					Name:  md3.EggPlantEggEnergyExchangeForBalance,
				},
			},
		},
		SettlementIntegralList: md.IntegralList{
			IncomeList: []md.TransferTypeNode{
				{
					Value: strconv.Itoa(md3.EggPlantEggEnergySignInSettlementTeamRewardForUserVirtualCoinFlow),
					Name:  md3.EggPlantEggEnergySignInSettlementTeamReward,
				},
				{
					Value: strconv.Itoa(md3.EggPlantTeamActiveCoinExchangeEggEnergyForUserVirtualCoinFlow),
					Name:  md3.EggPlantTeamActiveCoinExchangeEggEnergy,
				},
				{
					Value: strconv.Itoa(md3.EggPlantSettlementStarLevelDividendsForUserVirtualCoinFlow),
					Name:  md3.EggPlantSettlementStarLevelDividends,
				},
			},
			ExpendList: []md.TransferTypeNode{
				{
					Value: strconv.Itoa(md3.EggPlantSettlementEggEnergyExchangeTobeEggEnergyForUserVirtualCoinFlow),
					Name:  md3.EggPlantSettlementEggEnergyExchangeEggEnergy,
				},
			},
		},
	}

	resp := md.EggEnergyDetailResp{
		PersonEggEnergy:     personEggEnergy.Amount,
		TeamEggEnergy:       teamEggEnergy.Amount,
		TotalEggEnergy:      utils.Float64ToStr(totalEggEnergy),
		TotalEggEnergyPrice: utils.Float64ToStr(totalEggEnergyPrice),
		List:                list,
		TransferTypeList:    transferTypeList,
		Paginate: md.Paginate{
			Limit: req.Limit,
			Page:  req.Page,
			Total: total,
		},
	}

	e.OutSuc(c, resp, nil)
}

// EggPointDetails
// @Summary      蛋蛋星球-添加好友-蛋蛋积分明细(获取)
// @Tags         添加好友
// @Description  蛋蛋积分明细(获取)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Param    req    body	md.EggPointDetailsReq 	true  "分页类型必填"
// @Success      200   {object}   md.EggPointDetailsResp	"具体数据"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/v1/addFriend/eggPointDetails [POST]
func EggPointDetails(c *gin.Context) {
	val, exists := c.Get("user")
	if !exists {
		e.OutErr(c, e.ERR_USER_CHECK_ERR, nil)
		return
	}
	user, ok := val.(*model.User)
	if !ok {
		e.OutErr(c, e.ERR_USER_CHECK_ERR, nil)
		return
	}
	var req *md.EggPointDetailsReq
	if err1 := c.ShouldBindJSON(&req); err1 != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err1.Error())
		return
	}

	//1、查找 `OneCirclesPublicPlatoonBasicSetting` 基础设置
	energyBasicSettingDb := implement.NewEggEnergyBasicSettingDb(db.Db)
	eggEnergyBasicSetting, err := energyBasicSettingDb.EggEnergyBasicSettingGetOneByParams(map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	var coinID int
	isAsc := false
	if req.Kind == "1" { // 待结
		coinID = eggEnergyBasicSetting.PersonEggPointsCoinId
	} else { // 可用
		coinID = eggEnergyBasicSetting.TeamEggPointsCoinId
	}
	if req.Asc == "1" {
		isAsc = true
	}
	direction := 0
	switch req.Direction {
	case "1":
		direction = 1
	case "2":
		direction = 2
	default:
		direction = 0
	}
	transferType := 0
	switch req.Type {
	case "1":
		transferType = 1
	case "2":
		transferType = 2
	default:
		transferType = 0
	}
	flowDb := implement.NewUserVirtualCoinFlowDb(db.Db)
	flows, total, err := flowDb.UserVirtualCoinFlowFindByCoinAndUser(req.Page, req.Limit, coinID, user.Id, "", "", direction, isAsc, transferType)
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	var list = make([]map[string]string, 0)
	for _, flow := range flows {
		var tmp = map[string]string{
			"title":         flow.Title,
			"time":          flow.CreateAt,
			"type":          utils.IntToStr(flow.Direction),
			"value":         flow.Amount,
			"transfer_type": utils.IntToStr(flow.TransferType),
		}
		list = append(list, tmp)
	}
	var teamRewardSetting *md2.TeamRewardSettingStruct
	err = json.Unmarshal([]byte(eggEnergyBasicSetting.DirectPushReward), &teamRewardSetting)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
		return
	}
	virtualAmountDb := implement.NewUserVirtualAmountDb(db.Db)
	var totalEggPoints float64
	personEggPoints, err := virtualAmountDb.GetUserVirtualWalletBySession(user.Id, eggEnergyBasicSetting.PersonEggEnergyCoinId)
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	teamEggPoints, err := virtualAmountDb.GetUserVirtualWalletBySession(user.Id, eggEnergyBasicSetting.TeamEggEnergyCoinId)
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	totalEggPoints = utils.StrToFloat64(personEggPoints.Amount) + utils.StrToFloat64(teamEggPoints.Amount)
	coinDb := implement.NewVirtualCoinDb(db.Db)
	coin, err := coinDb.VirtualCoinGetOneByParams(map[string]interface{}{
		"key":   "id",
		"value": eggEnergyBasicSetting.PersonEggPointsCoinId,
	})
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	exchangeRatio, _ := decimal.NewFromString(coin.ExchangeRatio)
	totalEggPointsValue := decimal.NewFromFloat(totalEggPoints)
	totalEggPointsAmount := totalEggPointsValue.Div(exchangeRatio).String()

	transferTypeList := md.TransferTypeList{
		AvailableIntegralList: md.IntegralList{
			IncomeList: []md.TransferTypeNode{
				{
					Value: strconv.Itoa(md3.EggPlantWatchAdRewardPersonalActiveCoinForUserVirtualCoinFlow),
					Name:  md3.EggPlantWatchAdRewardPersonalActiveCoin,
				},
				{
					Value: strconv.Itoa(md3.EggPlantConsumeOrdersGiveEggEnergyToPersonForUserVirtualCoinFlow),
					Name:  md3.EggPlantConsumeOrdersGiveEggEnergyForPerson,
				},
			},
			ExpendList: []md.TransferTypeNode{
				{
					Value: strconv.Itoa(md3.EggPlantPersonalActiveCoinExchangeToBeEggEnergyForUserVirtualCoinFlow),
					Name:  md3.EggPlantPersonalActiveCoinExchangeToBeEggEnergy,
				},
			},
		},
		SettlementIntegralList: md.IntegralList{
			IncomeList: []md.TransferTypeNode{
				{
					Value: strconv.Itoa(md3.EggPlantWatchAdRewardTeamActiveCoinForUserVirtualCoinFlow),
					Name:  md3.EggPlantWatchAdRewardTeamActiveCoin,
				},
				{
					Value: strconv.Itoa(md3.EggPlantConsumeOrdersGiveEggEnergyToTeamForUserVirtualCoinFlow),
					Name:  md3.EggPlantConsumeOrdersGiveEggEnergyForTeam,
				},
			},
			ExpendList: []md.TransferTypeNode{
				{
					Value: strconv.Itoa(md3.EggPlantTeamActiveCoinExchangeToBeEggEnergyForUserVirtualCoinFlow),
					Name:  md3.EggPlantTeamActiveCoinExchangeToBeEggEnergy,
				},
			},
		},
	}

	resp := md.EggPointDetailsResp{
		PersonEggPoints:      personEggPoints.Amount,
		TeamEggPoints:        teamEggPoints.Amount,
		TotalEggPoints:       utils.Float64ToStr(totalEggPoints),
		TotalEggPointsAmount: totalEggPointsAmount,
		List:                 list,
		TransferTypeList:     transferTypeList,
		Paginate: md.Paginate{
			Limit: req.Limit,
			Page:  req.Page,
			Total: total,
		},
	}

	e.OutSuc(c, resp, nil)
}

func BasalRate(c *gin.Context) {
	//val, exists := c.Get("user")
	//if !exists {
	//	e.OutErr(c, e.ERR_USER_CHECK_ERR, nil)
	//	return
	//}
	//user, ok := val.(*model.User)
	//if !ok {
	//	e.OutErr(c, e.ERR_USER_CHECK_ERR, nil)
	//	return
	//}
	//
	////1、查找 `one_circles_green_energy_basic_setting` 基础设置
	//energyBasicSettingDb := implement.NewEggEnergyBasicSettingDb(db.Db)
	//eggEnergyBasicSetting, err := energyBasicSettingDb.EggEnergyBasicSettingGetOneByParams(map[string]interface{}{
	//	"key":   "is_open",
	//	"value": 1,
	//})
	//if err != nil {
	//	e.OutErr(c, e.ERR_DB_ORM, err.Error())
	//	return
	//}
	//var teamRewardSetting *md2.TeamRewardSettingStruct
	//err = json.Unmarshal([]byte(eggEnergyBasicSetting.DirectPushReward), &teamRewardSetting)
	//if err != nil {
	//	e.OutErr(c, e.ERR, err.Error())
	//	return
	//}
	//now := time.Now()
	//var oneRoundDuration = utils.StrToInt(teamRewardSetting.OneRoundDuration)
	//startTime := now.Add(-time.Hour * time.Duration(oneRoundDuration)).Format("2006-01-02 15:04:05")
	//
	//signInDb := implement.NewEggSignInDb(db.Db)
	//get, eggSignIn, err := signInDb.EggSignINGetOneByTimeAndUid(startTime, "", user.Id, 0)
	//if err != nil {
	//	e.OutErr(c, e.ERR_DB_ORM, err.Error())
	//	return
	//}
	//var signRewardEggEnergy string
	//var signCountdown string
	//if !get {
	//	signRewardEggEnergy = "0.00"
	//} else {
	//	signRewardEggEnergy = eggSignIn.EstimatePerSecondEggEnergyValue
	//	endTime := utils.TimeParseStd(eggSignIn.EndTime)
	//	if endTime.Before(now) {
	//		//TODO::证明此时上一轮签到已结束，暂时还没结算
	//		signRewardEggEnergy = "0.00"
	//	} else {
	//		duration := endTime.Sub(now)                  // 计算时间差值
	//		hours := duration / time.Hour                 // 获取小时部分
	//		minutes := duration % time.Hour / time.Minute // 获取分钟部分（先除去小时后再乘以60）
	//		seconds := int64(duration/time.Second) % 60
	//		signCountdown = fmt.Sprintf("%d:%d:%d", hours, minutes, seconds) //收益倒计时
	//	}
	//}
	//nowBasalRate := utils.StrToFloat64(signRewardEggEnergy) * 60 * 60 //每小时基础速率
	//
	////2、组装距状图数据
	//var signInRewards []*md.SignInRewardStruct
	//err = json.Unmarshal([]byte(oneCirclesGreenEnergyBasicSetting.SignInReward), &signInRewards)
	//if err != nil {
	//	e.OutErr(c, e.ERR, err.Error())
	//	return
	//}
	//var barChartYData, barChartXData []string
	//for _, v := range signInRewards {
	//	barChartYData = append(barChartYData, utils.Float64ToStrPrec8(utils.StrToFloat64(v.RewardValue)*60*60))
	//	var vipMemberStartNum, vipMemberEndNums int64
	//	vipMemberStartNum = utils.StrToInt64(v.VipMemberStartNums)
	//	vipMemberEndNums = utils.StrToInt64(v.VipMemberEndNums)
	//	var vipMemberStartNumStr, vipMemberEndNumsStr string
	//	if vipMemberStartNum >= 10000 && vipMemberStartNum < 100000 {
	//		vipMemberStartNum = vipMemberStartNum / 10000
	//		vipMemberStartNumStr = utils.Int64ToStr(vipMemberStartNum) + "万"
	//	} else if vipMemberStartNum >= 100000 && vipMemberStartNum < 1000000 {
	//		vipMemberStartNum = vipMemberStartNum / 100000
	//		vipMemberStartNumStr = utils.Int64ToStr(vipMemberStartNum) + "十万"
	//	} else if vipMemberStartNum >= 1000000 && vipMemberStartNum < 10000000 {
	//		vipMemberStartNum = vipMemberStartNum / 1000000
	//		vipMemberStartNumStr = utils.Int64ToStr(vipMemberStartNum) + "百万"
	//	} else if vipMemberStartNum >= 10000000 && vipMemberStartNum < 100000000 {
	//		vipMemberStartNum = vipMemberStartNum / 10000000
	//		vipMemberStartNumStr = utils.Int64ToStr(vipMemberStartNum) + "千万"
	//	} else if vipMemberStartNum >= 100000000 {
	//		vipMemberStartNum = vipMemberStartNum / 100000000
	//		vipMemberStartNumStr = utils.Int64ToStr(vipMemberStartNum) + "亿"
	//	} else {
	//		vipMemberStartNumStr = utils.Int64ToStr(vipMemberStartNum)
	//	}
	//
	//	if vipMemberEndNums >= 10000 && vipMemberEndNums < 100000 {
	//		vipMemberEndNums = vipMemberEndNums / 10000
	//		vipMemberEndNumsStr = utils.Int64ToStr(vipMemberEndNums) + "万"
	//	} else if vipMemberEndNums >= 100000 && vipMemberEndNums < 1000000 {
	//		vipMemberEndNums = vipMemberEndNums / 100000
	//		vipMemberEndNumsStr = utils.Int64ToStr(vipMemberEndNums) + "十万"
	//	} else if vipMemberEndNums >= 1000000 && vipMemberEndNums < 10000000 {
	//		vipMemberEndNums = vipMemberEndNums / 1000000
	//		vipMemberEndNumsStr = utils.Int64ToStr(vipMemberEndNums) + "百万"
	//	} else if vipMemberEndNums >= 10000000 && vipMemberEndNums < 100000000 {
	//		vipMemberEndNums = vipMemberEndNums / 10000000
	//		vipMemberEndNumsStr = utils.Int64ToStr(vipMemberEndNums) + "千万"
	//	} else if vipMemberEndNums >= 100000000 {
	//		vipMemberEndNums = vipMemberEndNums / 100000000
	//		vipMemberEndNumsStr = utils.Int64ToStr(vipMemberEndNums) + "亿"
	//	} else {
	//		vipMemberEndNumsStr = utils.Int64ToStr(vipMemberEndNums)
	//
	//	}
	//	barChartXData = append(barChartXData, vipMemberStartNumStr+"~"+vipMemberEndNumsStr)
	//}
	//
	////3、统计当前平台累计用户、距离下次减产还剩多少人
	//results, err := commDb.QueryNativeString(engine, "SELECT COUNT(*)AS total FROM user")
	//if err != nil {
	//	e.OutErr(c, e.ERR_DB_ORM, err.Error())
	//	return
	//}
	//userCount := utils.StrToInt64(results[0]["total"])
	//var nextReduceMembers string
	//for k, v := range signInRewards {
	//	if zhios_order_relate_utils.StrToInt64(v.VipMemberEndNums) >= userCount && userCount >= zhios_order_relate_utils.StrToInt64(v.VipMemberStartNums) {
	//		nextReduceMembers = utils.Int64ToStr(utils.StrToInt64(signInRewards[k+1].VipMemberStartNums) - userCount)
	//	}
	//}
	//e.OutSuc(c, map[string]interface{}{
	//	"now_basal_rate":      nowBasalRate,
	//	"sign_countdown":      signCountdown,
	//	"user_count":          userCount,
	//	"next_reduce_members": nextReduceMembers,
	//	"bar_chart_data": map[string]interface{}{
	//		"x_data": barChartXData,
	//		"y_data": barChartYData,
	//	},
	//}, nil)
	//return
	e.OutSuc(c, nil, nil)
}

// MyFans
// @Summary      蛋蛋星球-添加好友-粉丝团队-我的粉丝-团队加速速率(获取)
// @Tags         添加好友
// @Description  团队加速速率(获取)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Success      200   {object}   md.MyFansResp	"具体数据"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/v1/addFriend/myFans [GET]
func MyFans(c *gin.Context) {
	val, exists := c.Get("user")
	if !exists {
		e.OutErr(c, e.ERR_USER_CHECK_ERR, nil)
		return
	}
	user, ok := val.(*model.User)
	if !ok {
		e.OutErr(c, e.ERR_USER_CHECK_ERR, nil)
		return
	}
	// 找出公排中所有关联用户
	userRelateDb := implement.NewUserRelateDb(db.Db)
	total, err := userRelateDb.UserRelatesCountByParentUid(user.Id, 0)
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err)
		return
	}
	session := db.Db.NewSession()
	defer session.Close()
	now := time.Now()
	assistanceDb := implement.NewEggEnergyTeamAssistanceDb(db.Db)
	assistance, err := assistanceDb.EggEnergyTeamAssistanceGetOneBySession(session, user.Id, now.Format("2006-01-02"))
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err)
		return
	}
	var nowTeamAssistanceNum int
	var nowTeamAssistanceReward string
	if assistance != nil {
		nowTeamAssistanceNum = assistance.AssistedNum
		nowTeamAssistanceReward = assistance.RewardScore
	}
	resp := md.MyFansResp{
		NowTeamUser:             total,
		NowTeamAssistanceNum:    nowTeamAssistanceNum,
		NowTeamAssistanceReward: nowTeamAssistanceReward,
	}
	e.OutSuc(c, resp, nil)
	return
}

// MyFansUserList
// @Summary      蛋蛋星球-添加好友-粉丝团队-我的粉丝-在线好友列表(获取)
// @Tags         添加好友
// @Description  在线好友列表(获取)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Success      200   {object}   md.MyFansUserListResp	"具体数据"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/v1/addFriend/myFansUserList [GET]
func MyFansUserList(c *gin.Context) {
	page := c.DefaultQuery("page", "")
	limit := c.DefaultQuery("page_size", "20")

	now := time.Now()
	val, exists := c.Get("user")
	if !exists {
		e.OutErr(c, e.ERR_USER_CHECK_ERR, nil)
		return
	}
	user, ok := val.(*model.User)
	if !ok {
		e.OutErr(c, e.ERR_USER_CHECK_ERR, nil)
		return
	}

	energyBasicSettingDb := implement.NewEggEnergyBasicSettingDb(db.Db)
	eggEnergyBasicSetting, err := energyBasicSettingDb.EggEnergyBasicSettingGetOneByParams(map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	var teamRewardSetting *md2.TeamRewardSettingStruct
	err = json.Unmarshal([]byte(eggEnergyBasicSetting.DirectPushReward), &teamRewardSetting)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
		return
	}
	var oneRoundDuration = utils.StrToInt(teamRewardSetting.OneRoundDuration)
	startTime := now.Add(-time.Hour * time.Duration(oneRoundDuration)).Format("2006-01-02 15:04:05")
	relateDb := implement.NewUserRelateDb(db.Db)
	userRelates, err := relateDb.FindUserRelateByParentUid(user.Id, 1)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
		return
	}
	var userRelatesUids []string
	var userSignInMap = map[int64]interface{}{}
	var results []map[string]string
	if userRelates != nil {
		for _, userRelate := range *userRelates {
			userRelatesUids = append(userRelatesUids, utils.Int64ToStr(userRelate.Uid))
		}
		signInDb := implement.NewEggSignInDb(db.Db)

		eggSignIns, err1 := signInDb.EggSignInFindByTimeAndParams(startTime, "", 0, map[string]interface{}{
			"key":   "uid",
			"value": userRelatesUids,
		})
		if err1 != nil {
			e.OutErr(c, e.ERR_DB_ORM, err1.Error())
			return
		}
		for _, eggSignIn := range eggSignIns {
			userSignInMap[eggSignIn.Uid] = *eggSignIn
		}

		var sql string
		sql = "SELECT user.id AS uid, user.avatar AS avatar, user.nickname AS nickname, user_relate.invite_time AS invite_time " +
			"FROM `user_relate` " +
			"LEFT JOIN `user` ON user_relate.uid = user.uid " +
			"where user_relate.uid IN(%s)"
		sql += " ORDER BY user_relate.invite_time DESC LIMIT %d, %d;"
		sql = fmt.Sprintf(sql, strings.Join(userRelatesUids, ","), (utils.StrToInt64(page)-1)*utils.StrToInt64(limit), utils.StrToInt64(limit))
		results, err = db.QueryNativeString(db.Db, sql)
		if err != nil {
			e.OutErr(c, e.ERR_DB_ORM, err.Error())
			return
		}
	}

	var data []md.OneLineUserNode
	for _, v := range results {
		isOnline := false
		if userSignInMap[utils.AnyToInt64(v["uid"])] != nil {
			isOnline = true
		}
		data = append(data, md.OneLineUserNode{
			IsOnline:   isOnline,
			AvatarURL:  v["avatar_url"],
			Nickname:   v["nickname"],
			InviteTime: v["invite_time"],
		})
	}
	resp := md.MyFansUserListResp{
		List: data,
	}

	e.OutSuc(c, resp, nil)
	return
}

// NineDimensionalSpace
// @Summary      蛋蛋星球-添加好友-粉丝团队-九维空间(获取)
// @Tags         添加好友
// @Description  九维空间(获取)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Success      200   {object}   md.NineDimensionalSpaceResp	"具体数据"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/v1/addFriend/nineDimensionalSpace [GET]
func NineDimensionalSpace(c *gin.Context) {
	//1、 查找 `one_circles_public_platoon_basic_setting` 基础设置
	settingDb := implement.NewPublicPlatoonBasicSettingDb(db.Db)
	setting, err := settingDb.PublicPlatoonBasicSettingGetOneByParams(map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	if setting == nil {
		e.OutErr(c, e.ERR_NO_DATA, nil)
		return
	}
	val, exists := c.Get("user")
	if !exists {
		e.OutErr(c, e.ERR_USER_CHECK_ERR, nil)
		return
	}
	user, ok := val.(*model.User)
	if !ok {
		e.OutErr(c, e.ERR_USER_CHECK_ERR, nil)
		return
	}
	uid := user.Id

	var list []md.SpaceListNode
	for i := 1; i <= setting.SeveralRows; i++ {
		var tmpSql = fmt.Sprintf("SELECT COUNT(*)AS total FROM `public_platoon_user_relation` WHERE father_uid%d = %d", i, uid)
		tmpNativeString, _ := db.QueryNativeString(db.Db, tmpSql)
		nowUserCount := utils.StrToInt64(tmpNativeString[0]["total"])
		maxCount := math.Pow(float64(setting.SeveralTimes), float64(i))
		if nowUserCount > int64(maxCount) {
			nowUserCount = int64(maxCount)
		}
		list = append(list, md.SpaceListNode{
			Name:     utils.IntToStr(i) + "维",
			MaxCount: utils.Float64ToStr(maxCount),
			NowCount: utils.Int64ToStr(nowUserCount),
		})
	}

	//2、统计当前用户下多少人数
	sql := fmt.Sprintf("SELECT COUNT(*)AS total FROM `public_platoon_user_relation` WHERE father_uid1 = %d OR father_uid2= %d OR father_uid3= %d OR father_uid4= %d OR father_uid5= %d OR father_uid6= %d OR father_uid7= %d OR father_uid8= %d OR father_uid9= %d", uid, uid, uid, uid, uid, uid, uid, uid, uid)
	nativeString1, _ := db.QueryNativeString(db.Db, sql)
	hasUserCount := utils.StrToInt64(nativeString1[0]["total"])

	resp := md.NineDimensionalSpaceResp{
		SpaceTotalNums: "29523",
		SpaceNums:      utils.Int64ToStr(hasUserCount),
		SpaceList:      list,
		ALotOfNums:     "29523",
	}

	e.OutSuc(c, resp, nil)
	return
}
